'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _testcafeBrowserNatives = require('testcafe-browser-natives');

var _testcafeBrowserNatives2 = _interopRequireDefault(_testcafeBrowserNatives);

var _osFamily = require('os-family');

var _osFamily2 = _interopRequireDefault(_osFamily);

var _message = require('../../../warnings/message');

var _message2 = _interopRequireDefault(_message);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*eslint-disable no-undef*/
function getDataFromBrowser() {
    return { width: window.innerWidth, height: window.innerHeight, title: document.title };
}
/*eslint-disable no-undef*/

var INIT_SCRIPT = getDataFromBrowser.toString();

var BrowserProviderBase = function () {
    function BrowserProviderBase() {
        (0, _classCallCheck3.default)(this, BrowserProviderBase);

        // HACK: The browser window has different border sizes in normal and maximized modes. So, we need to be sure that the window is
        // not maximized before resizing it in order to keep the mechanism of correcting the client area size working. When browser is started,
        // we are resizing it for the first time to switch the window to normal mode, and for the second time - to restore the client area size.

        this.resizeCorrections = {};
    }

    BrowserProviderBase.prototype.calculateResizeCorrections = function () {
        var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(id) {
            var _ref2, width, height, title, _ref3, newWidth, newHeight;

            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            _context.next = 2;
                            return this.waitForConnectionReady(id);

                        case 2:
                            _context.next = 4;
                            return this.runInitScript(id, INIT_SCRIPT);

                        case 4:
                            _ref2 = _context.sent;
                            width = _ref2.width;
                            height = _ref2.height;
                            title = _ref2.title;
                            _context.next = 10;
                            return _testcafeBrowserNatives2.default.isMaximized(title);

                        case 10:
                            if (_context.sent) {
                                _context.next = 12;
                                break;
                            }

                            return _context.abrupt('return');

                        case 12:
                            _context.next = 14;
                            return _testcafeBrowserNatives2.default.resize(title, width, height, width, height);

                        case 14:
                            _context.next = 16;
                            return this.runInitScript(id, INIT_SCRIPT);

                        case 16:
                            _ref3 = _context.sent;
                            newWidth = _ref3.width;
                            newHeight = _ref3.height;
                            _context.next = 21;
                            return _testcafeBrowserNatives2.default.maximize(title);

                        case 21:
                            if (!(newWidth === width && newHeight === height)) {
                                _context.next = 23;
                                break;
                            }

                            return _context.abrupt('return');

                        case 23:

                            this.resizeCorrections[id] = { width: newWidth - width, height: newHeight - height };

                        case 24:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function calculateResizeCorrections(_x) {
            return _ref.apply(this, arguments);
        }

        return calculateResizeCorrections;
    }();

    BrowserProviderBase.prototype.resizeWindow = function () {
        var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(id, pageInfo, width, height) {
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            if (!_osFamily2.default.linux) {
                                _context2.next = 3;
                                break;
                            }

                            this.reportWarning(id, _message2.default.browserManipulationsNotSupportedOnLinux);
                            return _context2.abrupt('return');

                        case 3:

                            if (this.resizeCorrections[id]) {
                                width -= this.resizeCorrections[id].width;
                                height -= this.resizeCorrections[id].height;

                                delete this.resizeCorrections[id];
                            }

                            _context2.next = 6;
                            return _testcafeBrowserNatives2.default.resize(pageInfo.title, pageInfo.width, pageInfo.height, width, height);

                        case 6:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function resizeWindow(_x2, _x3, _x4, _x5) {
            return _ref4.apply(this, arguments);
        }

        return resizeWindow;
    }();

    BrowserProviderBase.prototype.takeScreenshot = function () {
        var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(id, pageInfo, screenshotPath) {
            return _regenerator2.default.wrap(function _callee3$(_context3) {
                while (1) {
                    switch (_context3.prev = _context3.next) {
                        case 0:
                            if (!_osFamily2.default.linux) {
                                _context3.next = 3;
                                break;
                            }

                            this.reportWarning(id, _message2.default.browserManipulationsNotSupportedOnLinux);
                            return _context3.abrupt('return');

                        case 3:
                            _context3.next = 5;
                            return _testcafeBrowserNatives2.default.screenshot(pageInfo.title, screenshotPath);

                        case 5:
                        case 'end':
                            return _context3.stop();
                    }
                }
            }, _callee3, this);
        }));

        function takeScreenshot(_x6, _x7, _x8) {
            return _ref5.apply(this, arguments);
        }

        return takeScreenshot;
    }();

    return BrowserProviderBase;
}();

exports.default = BrowserProviderBase;
module.exports = exports['default'];