'use strict';

exports.__esModule = true;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _lodash = require('lodash');

var _dedent = require('dedent');

var _dedent2 = _interopRequireDefault(_dedent);

var _clientFunctionBuilder = require('./client-function-builder');

var _clientFunctionBuilder2 = _interopRequireDefault(_clientFunctionBuilder);

var _replicator = require('./replicator');

var _runtime = require('../errors/runtime');

var _builderSymbol = require('./builder-symbol');

var _builderSymbol2 = _interopRequireDefault(_builderSymbol);

var _message = require('../errors/runtime/message');

var _message2 = _interopRequireDefault(_message);

var _observation = require('../test-run/commands/observation');

var _defineLazyProperty = require('../utils/define-lazy-property');

var _defineLazyProperty2 = _interopRequireDefault(_defineLazyProperty);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SelectorBuilder = function (_ClientFunctionBuilde) {
    (0, _inherits3.default)(SelectorBuilder, _ClientFunctionBuilde);

    function SelectorBuilder(fn, options, callsiteNames) {
        (0, _classCallCheck3.default)(this, SelectorBuilder);

        var builderFromSelector = fn && fn[_builderSymbol2.default];
        var builderFromPromiseOrSnapshot = fn && fn.selector && fn.selector[_builderSymbol2.default];
        var builder = builderFromSelector || builderFromPromiseOrSnapshot;

        builder = builder instanceof SelectorBuilder ? builder : null;

        if (builder) {
            fn = builder.fn;

            if (options === void 0 || (typeof options === 'undefined' ? 'undefined' : (0, _typeof3.default)(options)) === 'object') options = (0, _lodash.assign)({}, builder.options, options, { sourceSelectorBuilder: builder });
        }

        return (0, _possibleConstructorReturn3.default)(this, _ClientFunctionBuilde.call(this, fn, options, callsiteNames));
    }

    SelectorBuilder._defineNodeSnapshotDerivativeSelectorProperty = function _defineNodeSnapshotDerivativeSelectorProperty(obj, propName, fn) {
        (0, _defineLazyProperty2.default)(obj, propName, function () {
            var builder = new SelectorBuilder(function (fnArg) {
                /* eslint-disable no-undef */
                var selectorResult = selector();

                if (selectorResult && typeof selectorResult.then === 'function') return selectorResult.then(function (node) {
                    return fn(node, fnArg);
                });

                return fn(selectorResult, fnArg);
                /* eslint-enable no-undef */
            }, {
                dependencies: {
                    selector: obj.selector,
                    fn: fn
                }
            });

            return builder.getFunction();
        });
    };

    SelectorBuilder.prototype._getCompiledFnCode = function _getCompiledFnCode() {
        // OPTIMIZATION: if selector was produced from another selector and
        // it has same dependencies as source selector, then we can
        // avoid recompilation and just re-use already compiled code.
        var hasSameDependenciesAsSourceSelector = this.options.sourceSelectorBuilder && this.options.sourceSelectorBuilder.options.dependencies === this.options.dependencies;

        if (hasSameDependenciesAsSourceSelector) return this.options.sourceSelectorBuilder.compiledFnCode;

        var code = typeof this.fn === 'string' ? '(function(){return document.querySelectorAll(\'' + this.fn.replace(/'/g, "\\'") + '\');});' : _ClientFunctionBuilde.prototype._getCompiledFnCode.call(this);

        if (code) {
            return (0, _dedent2.default)('(function(){\n                    var __f$=' + code + '\n                    return function(){\n                        return window[\'%testCafeSelectorFilter%\'](__f$.apply(this, arguments), __dependencies$.__filterOptions$);\n                    };\n                 })();');
        }

        return null;
    };

    SelectorBuilder.prototype._createInvalidFnTypeError = function _createInvalidFnTypeError() {
        return new _runtime.ClientFunctionAPIError(this.callsiteNames.instantiation, this.callsiteNames.instantiation, _message2.default.selectorInitializedWithWrongType, (0, _typeof3.default)(this.fn));
    };

    SelectorBuilder.prototype._executeCommand = function _executeCommand(args, testRun, callsite) {
        var _this2 = this;

        var lazyPromise = _pinkie2.default.resolve();
        var resultPromise = null;

        // OPTIMIZATION: Selectors are executed lazily (once someone subscribes to their result).
        // It's especially useful for situations when the selector's result is passed to
        // an action, e.g.: `t.click(someSelector(42));` In that case, we create a new selector
        // inside this action, but we are not interested in the result of ongoing execution.
        var execute = function execute() {
            if (!resultPromise) {
                resultPromise = _ClientFunctionBuilde.prototype._executeCommand.call(_this2, args, testRun, callsite).then(function (result) {
                    return result ? _this2._decorateFunctionResult(result, args) : result;
                });
            }

            return resultPromise;
        };

        lazyPromise.then = function (onFulfilled, onRejected) {
            return execute().then(onFulfilled, onRejected);
        };
        lazyPromise.catch = function (onRejected) {
            return execute().catch(onRejected);
        };

        this._defineSelectorPropertyWithBoundArgs(lazyPromise, args);

        return lazyPromise;
    };

    SelectorBuilder.prototype.getFunctionDependencies = function getFunctionDependencies() {
        var dependencies = _ClientFunctionBuilde.prototype.getFunctionDependencies.call(this);
        var textFilter = this.options.textFilter;

        if (typeof textFilter === 'string') textFilter = new RegExp((0, _lodash.escapeRegExp)(textFilter));

        return (0, _lodash.assign)({}, dependencies, {
            __filterOptions$: {
                index: this.options.index || 0,
                textFilter: textFilter
            }
        });
    };

    SelectorBuilder.prototype._createTestRunCommand = function _createTestRunCommand(encodedArgs, encodedDependencies) {
        return new _observation.ExecuteSelectorCommand({
            instantiationCallsiteName: this.callsiteNames.instantiation,
            fnCode: this.compiledFnCode,
            args: encodedArgs,
            dependencies: encodedDependencies,
            visibilityCheck: !!this.options.visibilityCheck,
            timeout: this.options.timeout
        });
    };

    SelectorBuilder.prototype._validateNonNegativeNumberOption = function _validateNonNegativeNumberOption(name, value) {
        if (!(0, _lodash.isNil)(value) && (!(0, _lodash.isFinite)(value) || value < 0)) {
            var valueType = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);
            var actual = valueType === 'number' ? value : valueType;

            throw new _runtime.APIError(this.callsiteNames.instantiation, _message2.default.optionValueIsNotANonNegativeNumber, name, actual);
        }
    };

    SelectorBuilder.prototype._validateOptions = function _validateOptions(options) {
        _ClientFunctionBuilde.prototype._validateOptions.call(this, options);

        if (!(0, _lodash.isNil)(options.visibilityCheck)) {
            var visibilityCheckOptionType = (0, _typeof3.default)(options.visibilityCheck);

            if (visibilityCheckOptionType !== 'boolean') throw new _runtime.APIError(this.callsiteNames.instantiation, _message2.default.optionValueIsNotABoolean, 'visibilityCheck', visibilityCheckOptionType);
        }

        if (!(0, _lodash.isNil)(options.textFilter)) {
            var textFilterType = (0, _typeof3.default)(options.textFilter);

            if (textFilterType !== 'string' && !(0, _lodash.isRegExp)(options.textFilter)) throw new _runtime.APIError(this.callsiteNames.instantiation, _message2.default.optionValueIsNotAStringOrRegExp, 'textFilter', textFilterType);
        }

        this._validateNonNegativeNumberOption('timeout', options.timeout);
        this._validateNonNegativeNumberOption('index', options.index);
    };

    SelectorBuilder.prototype._getReplicatorTransforms = function _getReplicatorTransforms() {
        var transforms = _ClientFunctionBuilde.prototype._getReplicatorTransforms.call(this);

        transforms.push(new _replicator.SelectorNodeTransform());

        return transforms;
    };

    SelectorBuilder.prototype._defineSelectorPropertyWithBoundArgs = function _defineSelectorPropertyWithBoundArgs(obj, selectorArgs) {
        var _this3 = this;

        (0, _defineLazyProperty2.default)(obj, 'selector', function () {
            var builder = new SelectorBuilder(function () {
                return (/* eslint-disable no-undef */selector.apply(null, args)
                );
            } /* eslint-enable no-undef */
            , {
                dependencies: {
                    selector: _this3.getFunction(),
                    args: selectorArgs
                }
            });

            return builder.getFunction();
        });
    };

    SelectorBuilder.prototype._decorateFunctionResult = function _decorateFunctionResult(nodeSnapshot, selectorArgs) {
        this._defineSelectorPropertyWithBoundArgs(nodeSnapshot, selectorArgs);

        SelectorBuilder._defineNodeSnapshotDerivativeSelectorProperty(nodeSnapshot, 'getParentNode', function (node) {
            return node ? node.parentNode : node;
        });

        SelectorBuilder._defineNodeSnapshotDerivativeSelectorProperty(nodeSnapshot, 'getChildNode', function (node, idx) {
            return node ? node.childNodes[idx] : node;
        });

        SelectorBuilder._defineNodeSnapshotDerivativeSelectorProperty(nodeSnapshot, 'getChildElement', function (node, idx) {
            if (node.children) return node.children[idx];

            // NOTE: IE doesn't have `children` for non-element nodes =/
            var childNodeCount = node.childNodes.length;
            var currentElIdx = 0;

            for (var i = 0; i < childNodeCount; i++) {
                if (node.childNodes[i].nodeType === 1) {
                    if (currentElIdx === idx) return node.childNodes[i];

                    currentElIdx++;
                }
            }

            return null;
        });

        if (nodeSnapshot.classNames) nodeSnapshot.hasClass = function (name) {
            return nodeSnapshot.classNames.indexOf(name) > -1;
        };

        return nodeSnapshot;
    };

    return SelectorBuilder;
}(_clientFunctionBuilder2.default);

exports.default = SelectorBuilder;
module.exports = exports['default'];