'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _path = require('path');

var _promisify = require('../utils/promisify');

var _promisify2 = _interopRequireDefault(_promisify);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ensureDir = (0, _promisify2.default)(_mkdirp2.default);

var PNG_EXTENSION_RE = /(\.png)$/;

var Capturer = function () {
    function Capturer(baseScreenshotsPath, testScreenshotsPath, testEntry, connection) {
        (0, _classCallCheck3.default)(this, Capturer);

        this.enabled = !!baseScreenshotsPath;
        this.baseScreenshotsPath = baseScreenshotsPath;
        this.testScreenshotsPath = testScreenshotsPath;
        this.testEntry = testEntry;
        this.provider = connection.provider;
        this.id = connection.id;
    }

    Capturer._getFileName = function _getFileName(stepName) {
        return (stepName && stepName.replace(/\s|\\|\/|"|\*|\?|<|>|\|/g, '_') || 'page-load') + '.png';
    };

    Capturer._correctFilePath = function _correctFilePath(customPath) {
        return PNG_EXTENSION_RE.test(customPath) ? customPath : customPath + '.png';
    };

    Capturer.prototype._takeScreenshot = function () {
        var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(pageInfo, filePath) {
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            _context.next = 2;
                            return ensureDir((0, _path.dirname)(filePath));

                        case 2:
                            _context.next = 4;
                            return this.provider.takeScreenshot(this.id, pageInfo, filePath);

                        case 4:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function _takeScreenshot(_x, _x2) {
            return _ref.apply(this, arguments);
        }

        return _takeScreenshot;
    }();

    Capturer.prototype.captureAction = function () {
        var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(_ref3) {
            var pageInfo = _ref3.pageInfo;
            var stepName = _ref3.stepName;
            var customPath = _ref3.customPath;
            var fileName, filePath;
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            if (this.enabled) {
                                _context2.next = 2;
                                break;
                            }

                            return _context2.abrupt('return', null);

                        case 2:
                            fileName = Capturer._getFileName(stepName);
                            filePath = null;


                            if (customPath) {
                                filePath = (0, _path.join)(this.baseScreenshotsPath, Capturer._correctFilePath(customPath));

                                this.testEntry.path = this.baseScreenshotsPath;
                            } else filePath = (0, _path.join)(this.testScreenshotsPath, fileName);

                            _context2.next = 7;
                            return this._takeScreenshot(pageInfo, filePath);

                        case 7:

                            this.testEntry.hasScreenshots = true;

                            return _context2.abrupt('return', filePath);

                        case 9:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function captureAction(_x3) {
            return _ref2.apply(this, arguments);
        }

        return captureAction;
    }();

    Capturer.prototype.captureError = function () {
        var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(_ref5) {
            var pageInfo = _ref5.pageInfo;
            var stepName = _ref5.stepName;
            var screenshotRequired = _ref5.screenshotRequired;
            var filePath;
            return _regenerator2.default.wrap(function _callee3$(_context3) {
                while (1) {
                    switch (_context3.prev = _context3.next) {
                        case 0:
                            if (!(!screenshotRequired || !this.enabled)) {
                                _context3.next = 2;
                                break;
                            }

                            return _context3.abrupt('return', null);

                        case 2:
                            filePath = (0, _path.join)(this.testScreenshotsPath, 'errors', Capturer._getFileName(stepName));
                            _context3.next = 5;
                            return this._takeScreenshot(pageInfo, filePath);

                        case 5:
                            return _context3.abrupt('return', filePath);

                        case 6:
                        case 'end':
                            return _context3.stop();
                    }
                }
            }, _callee3, this);
        }));

        function captureError(_x4) {
            return _ref4.apply(this, arguments);
        }

        return captureError;
    }();

    return Capturer;
}();

exports.default = Capturer;
module.exports = exports['default'];