'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

var _testcafeBrowserNatives = require('testcafe-browser-natives');

var _utils = require('./commands/utils');

var _type = require('./commands/type');

var _type2 = _interopRequireDefault(_type);

var _message = require('../warnings/message');

var _message2 = _interopRequireDefault(_message);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BrowserManipulationQueue = function () {
    function BrowserManipulationQueue(browserConnection, screenshotCapturer, warningLog) {
        (0, _classCallCheck3.default)(this, BrowserManipulationQueue);

        this.commands = [];
        this.windowId = browserConnection.id;
        this.browserProvider = browserConnection.provider;
        this.screenshotCapturer = screenshotCapturer;
        this.warningLog = warningLog;
    }

    BrowserManipulationQueue.prototype._resizeWindow = function () {
        var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(pageInfo, width, height) {
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            _context.prev = 0;
                            _context.next = 3;
                            return this.browserProvider.resizeWindow(this.windowId, pageInfo, width, height);

                        case 3:
                            return _context.abrupt('return', _context.sent);

                        case 6:
                            _context.prev = 6;
                            _context.t0 = _context['catch'](0);

                            this.warningLog.addWarning(_message2.default.resizeError, _context.t0.message);
                            return _context.abrupt('return', null);

                        case 10:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this, [[0, 6]]);
        }));

        function _resizeWindow(_x, _x2, _x3) {
            return _ref.apply(this, arguments);
        }

        return _resizeWindow;
    }();

    BrowserManipulationQueue.prototype._resizeWindowToFitDevice = function () {
        var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(pageInfo, device, portrait) {
            var _getViewportSize, landscapeWidth, portraitWidth, width, height;

            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            _getViewportSize = (0, _testcafeBrowserNatives.getViewportSize)(device);
                            landscapeWidth = _getViewportSize.landscapeWidth;
                            portraitWidth = _getViewportSize.portraitWidth;
                            width = portrait ? portraitWidth : landscapeWidth;
                            height = portrait ? landscapeWidth : portraitWidth;
                            _context2.next = 7;
                            return this._resizeWindow(pageInfo, width, height);

                        case 7:
                            return _context2.abrupt('return', _context2.sent);

                        case 8:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function _resizeWindowToFitDevice(_x4, _x5, _x6) {
            return _ref2.apply(this, arguments);
        }

        return _resizeWindowToFitDevice;
    }();

    BrowserManipulationQueue.prototype._takeScreenshot = function () {
        var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(capture) {
            var fileName;
            return _regenerator2.default.wrap(function _callee3$(_context3) {
                while (1) {
                    switch (_context3.prev = _context3.next) {
                        case 0:
                            if (this.screenshotCapturer.enabled) {
                                _context3.next = 3;
                                break;
                            }

                            this.warningLog.addWarning(_message2.default.screenshotsPathNotSpecified);
                            return _context3.abrupt('return', null);

                        case 3:
                            _context3.prev = 3;
                            _context3.next = 6;
                            return capture();

                        case 6:
                            fileName = _context3.sent;
                            _context3.next = 9;
                            return (0, _testcafeBrowserNatives.generateThumbnail)(fileName);

                        case 9:
                            return _context3.abrupt('return', fileName);

                        case 12:
                            _context3.prev = 12;
                            _context3.t0 = _context3['catch'](3);

                            this.warningLog.addWarning(_message2.default.screenshotError, _context3.t0.message);
                            return _context3.abrupt('return', null);

                        case 16:
                        case 'end':
                            return _context3.stop();
                    }
                }
            }, _callee3, this, [[3, 12]]);
        }));

        function _takeScreenshot(_x7) {
            return _ref3.apply(this, arguments);
        }

        return _takeScreenshot;
    }();

    BrowserManipulationQueue.prototype.executePendingManipulation = function () {
        var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(driverMsg) {
            var _this = this;

            var command;
            return _regenerator2.default.wrap(function _callee4$(_context4) {
                while (1) {
                    switch (_context4.prev = _context4.next) {
                        case 0:
                            command = this.commands.shift();
                            _context4.t0 = command.type;
                            _context4.next = _context4.t0 === _type2.default.takeScreenshot ? 4 : _context4.t0 === _type2.default.takeScreenshotOnFail ? 7 : _context4.t0 === _type2.default.resizeWindow ? 10 : _context4.t0 === _type2.default.resizeWindowToFitDevice ? 13 : 16;
                            break;

                        case 4:
                            _context4.next = 6;
                            return this._takeScreenshot(function () {
                                return _this.screenshotCapturer.captureAction({
                                    pageInfo: driverMsg.pageInfo,
                                    stepName: _shortid2.default.generate(),
                                    customPath: command.path
                                });
                            });

                        case 6:
                            return _context4.abrupt('return', _context4.sent);

                        case 7:
                            _context4.next = 9;
                            return this._takeScreenshot(function () {
                                return _this.screenshotCapturer.captureError({
                                    pageInfo: driverMsg.pageInfo,
                                    stepName: _shortid2.default.generate(),
                                    screenshotRequired: true
                                });
                            });

                        case 9:
                            return _context4.abrupt('return', _context4.sent);

                        case 10:
                            _context4.next = 12;
                            return this._resizeWindow(driverMsg.pageInfo, command.width, command.height);

                        case 12:
                            return _context4.abrupt('return', _context4.sent);

                        case 13:
                            _context4.next = 15;
                            return this._resizeWindowToFitDevice(driverMsg.pageInfo, command.device, command.options.portraitOrientation);

                        case 15:
                            return _context4.abrupt('return', _context4.sent);

                        case 16:
                            return _context4.abrupt('return', null);

                        case 17:
                        case 'end':
                            return _context4.stop();
                    }
                }
            }, _callee4, this);
        }));

        function executePendingManipulation(_x8) {
            return _ref4.apply(this, arguments);
        }

        return executePendingManipulation;
    }();

    BrowserManipulationQueue.prototype.push = function push(command) {
        this.commands.push(command);
    };

    BrowserManipulationQueue.prototype.removeAllNonServiceManipulations = function removeAllNonServiceManipulations() {
        this.commands = this.commands.filter(function (command) {
            return (0, _utils.isServiceCommand)(command);
        });
    };

    return BrowserManipulationQueue;
}();

exports.default = BrowserManipulationQueue;
module.exports = exports['default'];